function [result, opt_hps] = run_ensemble_exp(data, scheme)
  addpath("./util");

  fprintf('============== Ensemble: Start grid search ==============\n');
  val_result = run_ensemble_grid(data, [], scheme);
  [tmp, opt_hps] = get_best(val_result);
  result = opt_hps;
  
  fprintf('============== Ensemble: Start training with the best hyper-parameters ==============\n');
  re_tr_hps = get_re_tr_hps(opt_hps);
  for idx = 1:size(re_tr_hps, 2)
    hps_str = re_tr_hps(idx);
    [lambda, k, repeat] = read_hps_str(hps_str, "sample");
    te_result = run_ensemble_grid( data, [], scheme, [lambda], [k], [repeat] );
    te_result = val_result;
    [te_res, tmp] = get_best(te_result);
    result = replace_hps_with_result(result, hps_str, te_res);
  end
end

function result = run_ensemble_grid(data, seed, scheme, lambda_list, k_list, repeat_list)
  if nargin < 2
    seed = 1;
  end
  if nargin < 3
    scheme = "user";
  end
  if nargin < 4
    lambda_list = [ 1, 2, 3, 4];
  end
  if nargin < 5
    k_list = [4 5 6];
  end
  if nargin < 6
    repeat_list = [1 2];
  end

  if length(seed) == 1
    rand('seed', seed);
    [tr_R, val_R] = split_data(data.R, 0.9);
    tr_data = struct();
    tr_data = setfield(tr_data, "R", tr_R);
    tr_data = setfield(tr_data, "Rt", val_R);
    tr_data = setfield(tr_data, "dim", data.dim);
    data = tr_data;
  end

  result = struct();
  for i=1:length(lambda_list)
    for j=1:length(k_list)
      for s = 1:length(repeat_list)
        field = sprintf('l%dk%drepeat%d', lambda_list(i), k_list(j), repeat_list(s));
        fprintf('====> start %s\n', field);
        lambda = 10^-lambda_list(i);
        k = 2^k_list(j);
	repeat = repeat_list(s);
	%fix subsample random seed
        rand('seed', 1);
        res = run_one_ensemble_new(data, lambda, k, repeat, scheme);
        result = setfield(result, field, res);
      end
    end
  end
end

function result = run_one_ensemble_new(data, lambda, k, repeat, scheme)
  result = struct();
  options = sprintf('-s 1 -t 10 -k %d -l %g -b 1 -n 8', k, lambda);
  m = ensemble_trainer(data.R, data.dim, 20, repeat, scheme, options);
  ret = pmf_predict_ranking(data.Rt, m.W, m.H, 10, data.R);
  r.ndcg = ret.ndcg; r.map = ret.map; r.auc = ret.auc; r.hlu = ret.hlu;
  r.pos_rank = ret.pos_rank;
  result = r;
  disp(r);
end

