function [result, opt_hps] = run_full_exp(data)
  addpath("./util");

  fprintf('============== Full: Start grid search ==============\n');
  val_result = run_full_grid(data);
  [tmp, opt_hps] = get_best(val_result);
  result = opt_hps;
  
  fprintf('============== Full: Start training with the best hyper-parameters ==============\n');
  re_tr_hps = get_re_tr_hps(opt_hps);
  for idx = 1:size(re_tr_hps, 2)
    hps_str = re_tr_hps(idx);
    [lambda, k, rho] = read_hps_str(hps_str, "full");
    te_result = run_full_grid( data, [], [lambda], [k], [1], [rho] );
    [te_res, tmp] = get_best(te_result);
    result = replace_hps_with_result(result, hps_str, te_res);
  end
end

function result = run_full_grid(data, seed, lambda_list, k_list, repeat_list, rho_list)
  if nargin < 2
    seed = 1;
  end
  if nargin < 3
    lambda_list = [ 1, 2, 3, 4];
  end
  if nargin < 4
    k_list = [4 5 6];
  end
  if nargin < 5
    repeat_list = [1 2];
  end
  if nargin < 6
    rho_list = [0 1 3 5];
  end

  if length(seed) == 1
    rand('seed', seed);
    [tr_R, val_R] = split_data(data.R, 0.9);
    tr_data = struct();
    tr_data = setfield(tr_data, "R", tr_R);
    tr_data = setfield(tr_data, "Rt", val_R);
    tr_data = setfield(tr_data, "dim", data.dim);
    data = tr_data;
  end

  result = struct();
  for i=1:length(lambda_list)
    for j=1:length(k_list)
      for s = 1:length(rho_list)
        %    rand('seed', 27);
        %W = rand(data.dim(1), 2^k_list(j));
        %H = rand(data.dim(2), 2^k_list(j));
        field = sprintf('l%dk%drho%d', lambda_list(i), k_list(j), rho_list(s));
        fprintf('====> start %s\n', field);
        rho = 2^-rho_list(s);
        lambda = 10^-lambda_list(i);
        k = 2^k_list(j);
        %res = run_one_full_new(data, W, H, lambda, k, rho);
        res = run_one_full_new(data, lambda, k, rho);
        result = setfield(result, field, res);
      end
    end
  end
end

%function result = run_one_full_new(data, W, H, lambda, k, rho)
function result = run_one_full_new(data, lambda, k, rho)
  result = struct();
  % full pu
  options = sprintf('-s 11 -t 10 -k %d -l %g -r %g -b 0 -n 8', k, lambda, rho);
  m = pmf_train(data.R, [], options);
  %m = pmf_train(data.R, [], W, H, options);
  ret = pmf_predict_ranking(data.Rt, m.W, m.H, 10, data.R);
  r.ndcg = ret.ndcg; r.map = ret.map; r.auc = ret.auc; r.hlu = ret.hlu;
  r.pos_rank = ret.pos_rank;
  result = r;
  disp(r);
end

