function [m] = ensemble_trainer(R, dim_of_R, nr_ensemblers, repeats, scheme, options)

if nargin < 4
	fprintf('m = ensemble_trainer(R, dim_of_R, nr_ensemblers, repeats [,scheme="%s", option="%s"])\n',...
			'user', '-s 1 -t 20 -l 0.1 -k 8 -b 0');
	fprintf('Use sample_trainer(R, dim_of_R, 1, 0, "", option="-s solvers -b 0 other_options") to perform other pu solvers\n');
	return 
end
if nargin < 5
	scheme = 'user';
end
if nargin < 6
	options = '-s 1 -t 20 -l 0.1 -k 8 -b 0';
end

users = dim_of_R(1);
items = dim_of_R(2);

if issparse(R)
	[I J V] = find(R);
	idx = find(I<=users & J <= items);
	R = [I(idx) J(idx) V(idx)];
end

nr_samples = repeats * size(R,1);

m.W = zeros(users, 0);
m.H = zeros(items, 0);

for s = 1: nr_ensemblers
	if strcmp(scheme,'user')
		newI = repmat(R(:,1),  repeats, 1);
		population = [1:items]'; 
		weights = ones(size(population))/length(population); 
		newJ = randsample([1:items]', nr_samples, true);
	elseif strcmp(scheme,'item_f')
		newI = repmat(R(:,1),  repeats, 1);
		population = [1:items]'; 
		weights = histc(R(:,2), population); weights = weights / sum(weights);
		newJ = randsample(population, nr_samples, true, weights);
	elseif strcmp(scheme,'item_w')
		newI = randsample([1:users]', nr_samples, true);
		population = [1:items]'; 
		weights = histc(R(:,2), population); weights = users - weights; weights = weights / sum(weights);
		newJ = randsample(population, nr_samples, true, weights);
	elseif strcmp(scheme,'item_s')
		newI = randsample([1:users]', nr_samples, true);
		population = [1:items]'; 
		weights = histc(R(:,2), population); weights(weights==0)=-1;weights = 1./weights; weights(weights<0) = 0; weights = weights / sum(weights);
		newJ = randsample(population, nr_samples, true, weights);
	elseif strcmp(scheme, 'uniform')
		population = [1:items]';
		weights = ones(size(population))/length(population);
		newI = randsample([1:users]', nr_samples, true);
		newJ = randsample([1:items]', nr_samples, true);
	else
		population = [1:items]';
		weights = ones(size(population))/length(population);
		newI = [];
		newJ = [];
		nr_samples = 0;
	end

	newV = zeros(nr_samples,1);
	newR = [R; [newI newJ newV]];

	tmpm = pmf_train(newR, [], options);
	m.W = [m.W tmpm.W];
	m.H = [m.H tmpm.H];
end
m.population = population;
m.weights = weights;

end
