%function [m] = sample_trainer_new(R, dim_of_R, W, H, repeats, scheme, options)
function [m] = sample_trainer_new(R, dim_of_R, repeats, scheme, options)

if nargin < 3
	fprintf('m = sample_trainer_new(R, dim_of_R, repeats [,scheme="%s", option="%s"])\n',...
			'user', '-s 1 -t 20 -l 0.1 -k 8 -b 1');
	fprintf(' schemes: user/item_f/item_w/item_s/uniform/');
	fprintf('Use sample_trainer(R, dim_of_R, 0, "", option="-s solvers -b 0 other_options") to perform other pu solvers\n');
	return 
end
if nargin < 4
	scheme = 'user';
end
if nargin < 5
	options = '-s 1 -t 20 -l 0.1 -k 8 -b 1';
end

users = dim_of_R(1);
items = dim_of_R(2);

if issparse(R)
	[I J V] = find(R);
	idx = find(I<=users & J <= items);
	R = [I(idx) J(idx) V(idx)];
end

nr_samples = repeats * size(R,1);
if strcmp(scheme,'user')
	newI = repmat(R(:,1),  repeats, 1);
	population = [1:items]'; 
	weights = ones(size(population))/length(population); 
	newJ = randsample([1:items]', nr_samples, true);
elseif strcmp(scheme,'item_f')
	newI = repmat(R(:,1),  repeats, 1);
	population = [1:items]'; 
	weights = histc(R(:,2), population); weights = weights / sum(weights);
	newJ = randsample(population, nr_samples, true, weights);
elseif strcmp(scheme,'item_w')
	newI = randsample([1:users]', nr_samples, true);
	population = [1:items]'; 
	weights = histc(R(:,2), population); weights = users - weights; weights = weights / sum(weights);
	newJ = randsample(population, nr_samples, true, weights);
elseif strcmp(scheme,'item_s')
	newI = randsample([1:users]', nr_samples, true);
	population = [1:items]'; 
	weights = histc(R(:,2), population); weights(weights==0)=-1;weights = 1./weights; weights(weights<0) = 0; weights = weights / sum(weights);
	newJ = randsample(population, nr_samples, true, weights);
elseif strcmp(scheme, 'uniform')
	population = [1:items]';
	weights = ones(size(population))/length(population);
	newI = randsample([1:users]', nr_samples, true);
	newJ = randsample([1:items]', nr_samples, true);
else
	population = [1:items]';
	weights = ones(size(population))/length(population);
	newI = [];
	newJ = [];
	nr_samples = 0;
end

newV = zeros(nr_samples,1);
newR = [R; [newI newJ newV]];

m = pmf_train(newR, [], options);
%m = pmf_train(newR, [], W, H, options);
m.population = population;
m.weights = weights;
m.scheme = scheme;
end
